' ==================== LCDhserinDriver.bas ====================

' Program runs on a Picaxe-20X2. It receives serial data from
' a master processor & displays it on an HD44780-type 16x2 LCD.

' === Constants ===
	symbol enable = C.7			' LCD enable pin connected to C.7
	symbol RegSel = C.4			' LCD RegSel pin connected to C.4

' === Variables ===
	symbol char   = b0			' character to be sent to LCD
	symbol LstChr = b1			' used to access hserin data
	symbol index  = b2			' used in for/next loop
	symbol newB.6 = pinC.1	' replacement pin for B.6 (hserin)
' Note:  newB.6 is a variable because its value can change

' === Directives ===
	#com 3									' specify COM port (AxGreen)
	#picaxe20X2							' specify compiler mode
	#no_data								' save time downloading
	#terminal off						' disable terminal window	
	
' =============================================================
' The following are custom characters with true descenders
' that we will use in place of the built-in versions.
' Note: Tese cannot be deleted without major program changes.
' =============================================================

' === Data ===

'	Lowercase "g"
	Data (%00000)
	Data (%00000)
	Data (%01111)	'  ****
	Data (%10001)	' *   *
	Data (%10001)	' *   *
	Data (%01111)	'  ****
	Data (%00001)	'     *
	Data (%01110)	'  ***

'	Lowercase "p"
	Data (%00000)
	Data (%00000)
	Data (%10110)	' * **
	Data (%11001)	' **  *
	Data (%10001)	' *   *
	Data (%11110)	' ****
	Data (%10000)	' *
	Data (%10000)	' *

'	Lowercase "q"
	Data (%00000)
	Data (%00000)
	Data (%01101)	'  ** *	
	Data (%10011)	' *  **	
	Data (%10001)	' *   *
	Data (%01111)	'  ****
	Data (%00001)	'     *	
	Data (%00001)	'     *

'	Lowercase "j"	
	Data (%00010)	'    *
	Data (%00000)	'    
	Data (%00110)	'	 **
	Data (%00010)	'    *	
	Data (%00010)	'    *
	Data (%00010)	'    *
	Data (%10010)	' *  *	
	Data (%01100)	'  **	

'	Lowercase "y"	
	Data (%00000)
	Data (%00000)
	Data (%10001)	' *	*
	Data (%10001)	' *   *	
	Data (%10001)	' *   *
	Data (%01111)	'  ****	
	Data (%00001)	'     *	
	Data (%01110)	'  ***
	
'	Bullet
	Data (%00000)
	Data (%01110)	'  ***
	Data (%11111)	' *****
	Data (%11111)	' *****
	Data (%11111)	' *****
	Data (%01110)	'  ***
	Data (%00000)
	Data (%00000)
	
'	Up-Arrow
	Data (%00100)	'   *
	Data (%01110)	'  ***
	Data (%10101)	' * * *
	Data (%00100)	'   *
	Data (%00100)	'   * 
	Data (%00100)	'   *
	Data (%00100)	'   *
	Data (%00000)
	
'	Down-Arrow
	Data (%00100)	'   *
	Data (%00100)	'   *
	Data (%00100)	'   *	
	Data (%00100)	'   *	
	Data (%10101)	' * * *
	Data (%01110)	'  ***
	Data (%00100)	'   *	
	Data (%00000)
	

' ==================== Begin Main Program =====================

	dirsB = %10111111				' configure portB (B.6 is hserin pin)
	dirsC = %10111111				' configure portC (C.6 is input only)
	hsersetup B4800_64, %111' setup for background serial receive
	
' === Initialize the LCD ===							 
	pause 200								' pause 200 mS for LCD initialization
	pinsB = 56							' setup for 8-bits,2 lines & 5X8 dots
 	pulsout enable,1				' send command
	char  = 12							' display on, cursor off
	gosub OutByte						' send instruction to LCD	
		
' === Load custom chars into CGRAM ===
	char = 64								' command: define custom characters
	low RegSel							' set up for command
	gosub OutByte2
	
	high RegSel							' set up for text	
	for index = 0 to 63			' install 8 custom characters
		read index, char
		gosub OutByte2
	next index

	char  = 1								' clear display and go home
	low RegSel							' this must be located here 	
	gosub OutByte2					' to "turn off" load custom chars
	
' === Main Program Loop - Receive data & display it on LCD ===
setfreq m64

do		
	if hserinflag = 1 then	' if background receive has occurred
		pause 320							' allow 40mS to receive max 20 chars
		LstChr = hserptr - 1	' get location of last byte received
		
	 	for ptr = 0 to LstChr	' for each scratchpad location,
	   	 char = @ptr				' fetch character
	   	 gosub OutByte			' process char and send it to LCD
	  	next ptr
	  	
		hserinflag = 0				' reset the background new data flag
		hserptr = 0						' reset the data write pointer
	endif
loop

' =========== End Main Program - Subroutines Follow ===========

OutByte:
	select case char					
		case 0 to 31					' commands
			low RegSel					' set up for command					
		case 128 to 207				' provides capability to move the
			low RegSel					' cursor to any on-screen location
		case 103,106,112,113	' "g", "j", "p", & "q"
			char = char - 103
			high RegSel					' set up for text			
		case 121							' "y"
			char = 4
			high RegSel					' set up for text
		else									' all other text
			high RegSel					' set up for text			
	endselect
	
Outbyte2:
	outpinsB = char					' load character onto outpinsB
	newB.6 = bit6						' substitute pinC.0 for pinB.6
	pulsout enable,1				' send data
	return