' ========================= MAXcount20X2.bas =========================

' This program runs on a PICAXE-20X2 & controls a MAX7219 LED display
' driver. The MAX7219 is connected to a 4-digit, 7-segment LED display.
' The program counts from 0 to 9999 in an infinite loop.

' === Constants ===
	symbol blank = 15		' used by MAX7219 to blank a digit

' Hardware interface to the MAX7219
	symbol load = C.2		' briefly pulse C.2 to transfer data to LEDs

' Register addresses for the MAX7219
	symbol decode =  9	' decode register; specify digits to decode
	symbol brite  = 10	' intensity (brightness) register; 15 = 100%
	symbol scan   = 11	' scan-limit register; specify how many digits
	symbol on_off = 12	' 1 = display on; 0 = display off 
		
' === Variables ===
	symbol cntr = w0		' used to count from 0 to 9999
	symbol ones = b2		' used to access the ones digit of cntr
	symbol tens = b3		' used to access the tens digit of cntr
	symbol hnds = b4		' used to access the hundreds digit of cntr
	symbol thos = b5		' used to access the thousands digit of cntr
	symbol val  = b6 		' data to be transmitted to the MAX7219

' === Directives ===
	#com 3							' specify the com port
	#picaxe 20X2				' specify the PICAXE processor
	#no_data						' speed up the download
	#no_table						' speed up the download
	#terminal off				' no terminal window
	
' ======================== Begin Main Program =========================

	setfreq m64
	dirsb = %11111111
	dirsc = %10111111
		
	hspisetup spimode00,spifast				' set up hspi

' Initialize MAX7219
	hspiout (scan,3)									' set scan limit for digits 0-3
	pulsout load,1
 	 
  	hspiout (brite,5) 							' set brightness to 5 (15 = 100%)
	pulsout load,1

  	hspiout (decode,15)							' set BCD decoding for digits 0-3
	pulsout load,1	
 	
  	hspiout (on_off,1)							' turn display on
	pulsout load,1

do
	for cntr = 0 to 9999							' main counting loop
	
		ones = cntr dig 0								' isolate 1st digit from right
		tens = cntr dig 1								' isolate 2nd digit from right
		hnds = cntr dig 2								' isolate 3rd digit from right
		thos = cntr dig 3								' isolate 4th digit from right

      if thos = 0 then							' thousands digit (1st from left)
      	val = blank
      else
      	val = thos 
      endif	
  		hspiout (1,val)
		pulsout load,1
	
		if hnds = 0 AND cntr < 100 then	' hundreds digit(2nd from left)
      	val = blank
      else
      	val = hnds
      endif	
  		hspiout (2,val)
		pulsout load,1
		
		if tens = 0 AND cntr < 10 then	' tens digit (3rd from left)
      	val = blank
      else
      	val = tens
      endif
  		hspiout (3,val)
		pulsout load,1

  		hspiout (4,ones)							' ones digit (4th from left)
		pulsout load,1
		
  	  	pause 500										' slow down the count a bit
  	next cntr 		
loop