' ========================= MAXcount08M.bas =========================
' This program uses a PICAXE-08M to manually shift-out SPI data to
' a MAX7219 LED display driver. The 7219 is connected to a 4-digit,
' 7-segment LED display. The program counts from 0 to 9999.

' === Constants ===
	symbol   blank = 15		' used by MAX7219 to blank a digit

' Hardware interface to the MAX7219 
	symbol 	 sData =  0 	' data out line to Max7219
	symbol 	 clock =  1 	' clock line
	symbol 	 sLoad =  2		' pulse briefly to load data onto LEDs		
	
' Register addresses for the MAX7219
	symbol  decode =  9		' decode register; specify digits to decode
	symbol   brite = 10		' intensity (brightness) register; 15 = 100%
	symbol  	scan = 11		' scan-limit register; specify # of LEDs
	symbol  on_off = 12		' 1 = display on; 0 = display off
	
' === Variables =====
	symbol outByte = b0 	' data byte to be transmitted to the LED
	symbol  maxReg = b1		' MAX register that receives the data byte
	symbol  forMax = w0		' We're breaking a rule here! (See text)
	symbol   index = b2		' used in for-next loop


	symbol 	  ones = b3		' used to access the ones digit of cntr
	symbol 	  tens = b4		' used to access the tens digit of cntr
	symbol 	  hnds = b5		' used to access the hundreds digit of cntr
	symbol 	  thos = b6		' used to access the thousands digit of cntr
	symbol	  tths = b7 	' just because bintoascii requires it
	symbol 	  cntr = w4		' used to count from 0 to 9999
		
' === Directives ====
	#com 3	 							' specify com port
	#picaxe 08M	 				  ' specify PICAXE processor
	#terminal off					' disable terminal window
	
' ======================= Begin Main Program ========================
	setfreq m8
	dirs = %00010111

' Initialize MAX7219
	maxReg  = scan									  ' set scan limit for digits 0-3
	outByte = 3
	gosub shout16

	maxReg  = brite										' set brightness to 5 (15 = 100%)
	outByte = 5	
	gosub shout16

	maxReg  = decode								  ' set BCD decoding for digits 0-3
	outByte = 15	
	gosub shout16
 	
	maxReg  = on_off					 				' turn display on
	outByte =	1
	gosub shout16

do
	for cntr = 0 to 9999							' main counting loop
		bintoascii cntr, tths,thos,hnds,tens,ones
		ones = ones - 48								' isolate 1st digit from right
		tens = tens - 48								' isolate 2nd digit from right
		hnds = hnds - 48								' isolate 3rd digit from right
		thos = thos - 48							 	' isolate 4th digit from right

    if thos = 0 then						 		' thousands digit (1st from left)
      outByte = blank
    else
      outByte  = thos 
    endif
    maxReg = 1	
  	gosub shout16
	
		if hnds = 0 AND cntr < 100 then	' hundreds digit(2nd from left)
      outByte  = blank
    else
      outByte  = hnds
    endif
    maxReg = 2	
  	gosub shout16
		
		if tens = 0 AND cntr < 10 then	' tens digit (3rd from left)
      outByte  = blank
    else
      outByte  = tens
    endif
    maxReg = 3
  	gosub shout16

		outByte = ones
		maxReg  = 4
  	gosub shout16								 		' ones digit (4th from left)
  next cntr 		
loop

' ============== End Main Program - Subroutines follow ==============

shout16:	
	for index = 15 to 0 step -1			 ' MAX7219 requires a 16-bit word
		if bit15 = 1 then							 ' set sdata to correspond to bit15
		   high sData
		else
		   low  sData
		endif
		pulsout clock, 2  				 		 ' briefly pulse the clock line 
		forMax = forMax * 2 					 ' shift char left for next MSB 
	next index
	
	pulsout sLoad, 2 								 ' briefly pulse the load line
	return