' =============================== MPDOS.bas ===============================

' Program implements an interrupt-driven "operating system" for the EG MPD.

' === Constants ===
	symbol 	   sp =  32					' blank space
	symbol 	   ua =  92					' up arrow
	symbol 	   da =  94					' down arrow
	symbol 	   ud = 123					' up & down arrow
	symbol 	   la = 127					' left arrow
	symbol 	   ra = 126					' right arrow
	symbol 	   lr = 125					' left & right arrow
	symbol enable = C.7					' LCD enable pin
	symbol RegSel = C.4					' LCD RegSel pin
	
' === Variables ===
	symbol 	 char = b0					' character to be sent to LCD
	symbol 	 junk = b1					' used in key de-bouncing
	symbol 	  key = w1					' ADC value of key-press
	symbol  index = b4					' used in for/next loop
	symbol  first = b5					' table pointer for first char in line 2
	symbol 	 last = b6					' table pointer for last char in line 2	
	symbol    plr = b7					' prompt for left & right arrows in line 2
	symbol    pud = b8					' prompt for up & down arrows in line 2
' Note:  The following are variables because their value can change
	symbol newB.4 = outpinC.3		' replacement pin for B.4
	symbol newB.6 = outpinA.0		' replacement pin for B.6 (hserin pin)

' === Directives ===
	#com 4											' specify COM port
	#picaxe 20X2								' specify compiler mode
	#terminal off								' disable terminal window	
	
' =========================================================================
' The following are custom characters with true descenders that we use
' in place of the built-in versions. These characters cannot be deleted
' without major program changes. However, once the program has been run,
' the data definitions & load custom characters code can be commented out.
' =========================================================================

' === Data ===
'	Lowercase "g"
	Data (%00000)
	Data (%00000)
	Data (%01111)	'  ****
	Data (%10001)	' *   *
	Data (%10001)	' *   *
	Data (%01111)	'  ****
	Data (%00001)	'     *
	Data (%01110)	'  ***

'	Lowercase "p"
	Data (%00000)
	Data (%00000)
	Data (%10110)	' * **
	Data (%11001)	' **  *
	Data (%10001)	' *   *
	Data (%11110)	' ****
	Data (%10000)	' *
	Data (%10000)	' *

'	Lowercase "q"
	Data (%00000)
	Data (%00000)
	Data (%01101)	'  ** *	
	Data (%10011)	' *  **	
	Data (%10001)	' *   *
	Data (%01111)	'  ****
	Data (%00001)	'     *	
	Data (%00001)	'     *

'	Lowercase "j"	
	Data (%00010)	'    *
	Data (%00000)	'    
	Data (%00110)	'	 **
	Data (%00010)	'    *	
	Data (%00010)	'    *
	Data (%00010)	'    *
	Data (%10010)	' *  *	
	Data (%01100)	'  **	
	
'	Up-Arrow			' "\" = char 92
	Data (%00100)	'   *
	Data (%01110)	'  ***
	Data (%10101)	' * * *
	Data (%00100)	'   *
	Data (%00100)	'   * 
	Data (%00100)	'   *
	Data (%00100)	'   *
	Data (%00000)
	
'	Up-Dn Arrows	' "{" = char 123	
	Data (%00100)	'   *
	Data (%01110)	'  ***
	Data (%10101)	' * * *
	Data (%00100)	'   *	
	Data (%10101)	' * * *
	Data (%01110)	'  ***	
	Data (%00100)	'   *	
	Data (%00000)
	
'	Down-Arrow		' "^" = char 94
	Data (%00100)	'   *
	Data (%00100)	'   *
	Data (%00100)	'   *	
	Data (%00100)	'   *	
	Data (%10101)	' * * *
	Data (%01110)	'  ***
	Data (%00100)	'   *	
	Data (%00000)
	
	'Lt-Rt Arrows	' "}" = char 125
	Data (%00100)	'   * 
	Data (%00010)	'    *
	Data (%11111)	' *****
	Data (%00000)
	Data (%11111)	' *****
	Data (%01000)	'  *
	Data (%00100)	'   *
	Data (%00000)
	
' =========== Table ===========
	Table  0,(" Select Program ")
	Table 16,("  Run           ")	
	Table 32,("   BeamMeUp     ")
	Table 48,("   Cloaking     ")
	Table 64,("   DeathRay     ")
	Table 80,("   Doomsday     ")
	
' ========================== Begin Main Program ===========================

setfreq m16										' set frequency to 16MHz
dirsB = %11101111							' configure pinB.4 as input
dirsC = %10111111							' configure pinC.6 as input
pullup  %00000010							' enable internal pullup resistor on C.6	
adcsetup = %0000000001000000	' see "adcsetup" in Manual
compsetup  %0101010101, 0			' see "compsetup"  in Manual
setintflags 16,16							' see "setintflags" in Manual

first = 32										' initialize table pointer: char  1, ln 2
last  = first + 13						' initialize table pointer: char 14, ln 2
plr = sp											' initialize lr prompt: list top, blank
pud = da											' initialize ud prompt: list top, dn-arrow

' === Initialize the LCD ===								 
pause 400											' pause 200 mS for LCD initialization
char = 56											' setup for 8-bits,2 lines & 5X8 dots
low RegSel										' set up for command
gosub OutByte2
char = 12											' display on, cursor off
gosub OutByte	
		
' === Load custom chars ===
char = 64											' command: define custom characters
low RegSel										' set up for command
gosub OutByte2

high RegSel										' setup for text		
for index = 0 to 63						' install 8 custom characters
	read index, char
	gosub OutByte2
next index
	
char = 1											' clear display & go home	
gosub OutByte									' (also turns off "load cust chars")
wait 1

key = 0												' initialize key to "no key-press"
plr = sp											' initialize left-rt pointer to blank
pud = sp											' initialize up-down pointer to blank

' === Main Program Loop ===
do
  char = 128:gosub OutByte		' move cursor to start of line one	
 	for index = 0 to 15					' send line one to LCD
 	  readtable index, char
 	  gosub OutByte
 	next index
 	
 	char = 192:gosub OutByte		' move cursor to start of line two
  for index = first to last		' send line two to LCD
 	  readtable index, char
 	  gosub OutByte
 	next index	
 	char = plr:gosub OutByte
 	char = pud:gosub OutByte
 				
 	select case key
 		case 0										' no key-press, so do nothing
 		case "A"
 		 	char = 128:gosub OutByte
 			for index = 1 to 16			' clear line one
 				char = " "
 				gosub OutByte
 			next index		
 			select case first				' choose a subroutine
 				case 32
 					gosub Program1
 				case 48
 					gosub Program2
 				case 64
 					gosub Program3
 				case 80
 					gosub Program4
 			end select	
 		case "D"
 			if first < 80 then			' not at bottom,so scroll down
 				first = first + 16
 				last = last + 16
 				sound C.1,(80,20)			' "good" beep
 			else
 				sound C.1,(60,50)			' "bad" beep
 			endif
 		case "U"
 			if first > 32 then 			' not at top, so scroll up
 				first = first - 16
 				last  = last - 16
 				sound C.1,(80,20)			' "good" beep
 			else
 			 	sound C.1,(60,50)			' "bad" beep
 			endif
 		else											' can't scroll past either end of list
 			sound C.1,(60,50)				' "bad" beep
 	end select
 	key = 0											' reset key to "null" (no key-press) 												
 	select case first
		case 32										' at top of list, only show down-arrow
 			plr = sp:pud = da
 		case 80										' at bottom of list, only show up-arrow
 			plr = sp:pud = ua
 		else											' in middle of list, show both arrows
 			plr = sp:pud = ud
 	end select
 
  char = 192:gosub OutByte		' move cursor to start of line two
  for index = first to last		' send line two to LCD
 	  readtable index, char
 	  gosub OutByte
 	next index	
 	char = plr:gosub OutByte		' include appropriate prompts
 	char = pud:gosub OutByte
loop

' ================= End Main Program - Subroutines Follow =================

' === Interrupt Subroutine - get key-press ===
interrupt:
	setintflags off							' disable interrupts
	compsetup 0,0								' disable comp so ADC is available	
	pause 10										' de-bounce key
	readadc10 6, key						' get ADC value
		
	wait_for_release:						' wait for key to be released	
	readadc 6, junk
	if junk > 5 then wait_for_release
	
	select case key							' decode key
		case < 364 : key = 65			' A
		case < 379 : key = 82			' R
		case < 395 : key = 68			' D
		case < 422 : key = 76			' L
		case < 452 : key = 48			' 0
		case < 476 : key = 57			' 9
		case < 502 : key = 85			' U
		case < 542 : key = 66			' B
		case < 588 : key = 56			' 8
		case < 629 : key = 55			' 7
		case < 674 : key = 54			' 6
		case < 733 : key = 53			' 5
		case < 804 : key = 52			' 4
		case < 884 : key = 51			' 3
		case < 976 : key = 50			' 2
		else       : key = 49			' 1
	end select
	
	compsetup  %0101010001, 0		' re-enable Comparator 1
	compflag = 0								' clear the interrupt flag
	setintflags 16,16						' re-enable interrupts	
	return

' === OutByte Subroutine - Send char to LCD ===
OutByte:
	high RegSel									'default value
	
	select case char					
		case 103,106,112,113			' "g", "j", "p", & "q"
			char = char - 103				' use custom chars with descenders
		case 92,94								' up-arrow & down-arrow
			char = char - 88	
		case 123,125							' up-down arrow & left-right arrow
			char = char - 118				
		case 32 to 127						' all other printable text
			' do nothing				
		else											' all other chars are commands
			low RegSel							' so set up for them		
	endselect
	
	Outbyte2:
	outpinsB = char							' load character onto outpinsB
	newB.4 = bit4								' substitute outpinC.3 for pinB.4
	newB.6 = bit6								' substitute outpinA.0 for pinB.6
	pulsout enable,2						' send data	
	return
	
' === Program1 Subroutine ===
Program1:
key = 0												' reset key to "null" (no key-press)
char = 192:gosub OutByte			' move cursor to start of line two	 
 	
for index = 16 to 21					' and display the choice
	readtable index, char
 	gosub OutByte
next index

for index = 35 to 44
	readtable index, char
 	gosub OutByte
next index

do
	sound C.1,(30,10)						' make a silly sound
	sound C.1,(20,10)
loop until key = "B"					' "B" key returns to main program loop

key = 0												' reset key to "null" (no key-press)
return 

' === Program2 Subroutine ===
Program2:
key = 0												' reset key to "null" (no key-press)	
char = 192:gosub OutByte			' move cursor to start of line two

for index = 16 to 21					' and display the choice
	readtable index, char
	gosub OutByte
next index
 	
for index = 51 to 60
	readtable index, char
	gosub OutByte
next index
 	
do
	sound C.1,(16,2)						' make a silly sound
 	sound C.1,(15,2)	 	
loop until key = "B"					' "B" key returns to main program loop

key = 0												' reset key to "null" (no key-press)
return 	

' === Program3 Subroutine ===
Program3:
key = 0												' reset key to "null" (no key-press)		
char = 192:gosub OutByte			' move cursor to start of line two 
	
for index = 16 to 21					' and display the choice
	readtable index, char
	gosub OutByte
next index
	
for index = 67 to 76
	readtable index, char
	gosub OutByte
next index
 	
do
 	sound C.1,(2,30)						' make a silly sound 
	sound C.1,(1,30)
loop until key = "B"					' "B" key returns to main program loop

key = 0												' reset key to "null" (no key-press)
return

' === Program4 Subroutine ===
Program4:
key = 0												' reset key to "null" (no key-press)			
char = 192:gosub OutByte			' move cursor to start of line two

for index = 16 to 21					' and display the choice
	readtable index, char
	gosub OutByte
next index

for index = 83 to 92
	readtable index, char
	gosub OutByte
next index
 	
do
	sound C.1,(10,15)						' make a silly sound
 	sound C.1,( 5,15)	 		
loop until key = "B"					' "B" key returns to main program loop

key = 0												' reset key to "null" (no key-press)
return