' ====================== TestInterrupt.bas =======================

'    This program tests the ADC keypad interrupt on the EG MPD.

' === Constants ===
	symbol horn = C.1								' speaker or piezo on C.1
	symbol LED  = C.5								' LED on C.5
	
' === Variables ===
	symbol char = b0								' character to be sent to LCD
	symbol junk = b1								' throwaway character
	symbol  key = w1								' ADC key-press value

' === Directives ===
	#com 4													' specify COM port
	#picaxe 20X2										' specify compiler mode
	#no_data												' save time downloading
	#no_table												' save time downloading
	#terminal 19200									' open terminal window	

' ====================== Begin Main Program ======================

setfreq m16												' set frequency to 16MHz
dirsB = %11101111									' configure pinB.4 as input
dirsC = %10111111									' configure pinC.6 as input
pullup  %00000010									' enable internal pullup  on C.6
adcsetup = %0100000001000000			' see "sdcsetup" in Manual
compsetup  %0101010101, 0					' see "compsetup" in Manual
setintflags %00010000,%00010000		' see "setintflags" in Manual
	
do
	toggle LED											' toggle LED slowly
	wait 8
loop

' ============ End Main Program - Subroutines Follow =============

interrupt:
	setintflags off									' disable comparator interrupt	
	compsetup 0,0										' disable comp to use ADC
	pause 10												' debounce the key-press
	readadc10 6, key								' get ADC value
		
	wait_for_release:								' wait for key to be released	
	readadc 6, junk
	if junk > 5 then wait_for_release
	
	select case key									' decode key
		case < 364 : char = 65				' A
		case < 379 : char = 82				' R
		case < 395 : char = 68				' D
		case < 422 : char = 76				' L
		case < 452 : char = 48				' 0
		case < 476 : char = 57				' 9
		case < 502 : char = 85				' U
		case < 542 : char = 66				' B
		case < 588 : char = 56				' 8
		case < 629 : char = 55				' 7
		case < 674 : char = 54				' 6
		case < 733 : char = 53				' 5
		case < 804 : char = 52				' 4
		case < 884 : char = 51				' 3
		case < 976 : char = 50				' 2
		else       : char = 49				' 1
	end select
	
	sertxd ("key = ",char,cr,lf)		' display key-press on Terminal
	sound horn,(50,20)							' beep
	 
	compsetup  %0101010001, 0				' re-enable Comparator 1
	compflag = 0										' clear the interrupt flag
	setintflags %00010000,%00010000	' re-enable comparator interrupt
	return