' =============== EG18 EZ0toLEDs.bas ===============

' Program uses a MaxSonar EZ0 to measure distance to
' nearest object & sends results to the EG SPI LED.

' === Constants ===
	symbol EZ0   =   6			' EZ0 analog on ADC6 (B.4)
	symbol LED   = A.0			' debugging LED on A.0
	symbol load  = C.2			' 7219 "Load" pin
	symbol blank =  15			' used to blank an LED

' Register addresses for the MAX7219
	symbol decode =  9			' decode register
	symbol brite  = 10			' intensity register
	symbol scan   = 11			' scan-limit register
	symbol on_off = 12			' 1 = display on; 0 = off 

' === Variables ===
	symbol dist = w0				' data from EZ0
	symbol ones = b2				' for LED display
	symbol tens = b3				' for LED display

' === Directives ===
	#com 3									' specify serial port
	#PICAXE 20X2						' specify processor
	#no_data								' save time downloading
	#no_table								' save time downloading
	#terminal off						' disable terminal window
		
' =============== Begin Main Program ===============

	setfreq m8
	dirsb = %11101111							' EZ0 is B.4 (ADC6)
	dirsc = %10111111							' C.6 is input only	
	adcsetup = %0000000001000000	' enable ADC6 (B.4)
	hspisetup spimode00,spislow		' set up hspi

' Initialize MAX7219
	hspiout (scan,3)				' set scan for digits 0-3
	pulsout load,1
	
	hspiout (brite,5)				' set brightness to 5
	pulsout load,1
	
	hspiout (decode,15)			' set BCD for digits 0-3
	pulsout load,1
		
	hspiout (on_off,1)			' turn display on
	pulsout load,1

do
  high LED								' for debugging
  wait 1
  readadc10 EZ0,dist			' get EZ0 raw data
  dist = dist / 2					' convert to inches
  dist = dist + 2					' correct for power filter 
  tens = dist / 10				' isolate the tens digit
  ones = dist // 10				' isolate the ones digit
  if tens = 0 then				' for zero-blanking
  	 tens = blank
  endif
  
' Send data to the four LED digits
	hspiout (1,blank)
	pulsout load,1
	
	hspiout (2,tens)
	pulsout load,1
	
	hspiout (3,ones)
	pulsout load,1
	
	hspiout (4,blank)
	pulsout load,1
	
  low LED
  wait 1
loop