' =================== HailOctavius.bas ===================

'	This program runs on the Octavius PICAXE-40X2 at 8MHz.
' Initially, it waits for a command from the IR remote,
' and then Octavius begins to explore his environment,
' avoiding obstacles in the process. At any point, the
' IR Remote can be used to override Octavius' behavior
' if he is about to to damage to himself or others :-)

' === Constants ===
	symbol fromIR =  D.1		' input from IR module on D.1
	symbol   toIR =  B.7		' output to  IR module on B.7
	symbol   spdL	=  C.2		' PWM2 (left  motor) on C.2
	symbol   spdR =  C.1		' PWM1 (right motor) on C.1
	symbol   dirL =  B.4		' left  motor direction on B.4
	symbol   dirR =  B.3		' right motor direction on B.3
	symbol    EZ1 =    7		' EZ1 analog input on ADC7 (A.7)
	symbol   LEDs =  B.6		' output to LED display on B.6
	symbol  blank =   15		' used to "blank" an LED digit
	symbol speedL =  320		' PWMOUT: speed / 4 = PerCent
	symbol speedR =  340		' 300 (75%) is the minimum val
	symbol   abit = 1500		' used in turns
	
' speedL and speedR are slightly different to compensate
' for small difference in motor speeds at same PWM duty.
	
' === Variables ===
  symbol dist = w0				' data from EZ1
  symbol cmnd = b2				' value of received command
	symbol ones = b3				' needed for LED display
	symbol tens = b4				' needed for LED display
	
' Note:  IRflag is a variable becuase its value changes
	symbol IRflag = pinD.1	' input from IR module on D.1
 
' === Directives ===
	#com 3									' specify com port
	#picaxe 40X2						' specify processor
	#no_data								' reduce download time
	#no_table								' reduce download time
	#terminal off						' disable terminal window
	
' ================== Begin Main Program ==================

setfreq m8
ADCsetup = 7							' ADC input on channels 0-7

top:
pwmout spdL,99,0					' stop both motors
pwmout spdR,99,0

tarry:
if IRflag = 0 then tarry	' wait for "go" from IR remote

do
	if IRflag = 1 then
		 gosub getcmnd				' get cmnd if available
	endif
	if cmnd = 20 then				' "mute" cmnd exits loop and
		exit									' stops exploratory behavior
	endif
	pause 400								' slow down
	gosub getdist						' look for obstacle
	if dist < 25 then				' if its closer than 25 inches
		 gosub left						' then turn away
		 gosub frwrd
	endif
	pause 400								' slow down some more
loop

goto top									' only when "mute" cmnd is sent

'	======== End Main Program - Subroutines follow =========

getcmnd: 									' get IR command
pulsout toIR,10						'50uS "send it" pulse	
serin fromIR,N2400_8,cmnd
select case cmnd					' move accordingly
  case 16									' Channel +
  	gosub frwrd
  case 17									' Channel -
  	gosub rvrse
  case 18									' Volume +
  	gosub right
  case 19									' Volume -
  	gosub left
  case 20									' Mute
  	gosub stopp
end select
return 

getdist: 									' get distance to  obstacle
readadc10 EZ1,dist				' get EZ1 raw data
dist = dist / 2						' convert to inches
dist = dist + 2						' correct for power filter
tens = dist / 10					' isolate the tens digit
ones = dist // 10					' isolate the ones digit
if tens = 0 then					' for zero-blanking
   tens = blank
endif
serout LEDs, N2400_8, (blank,tens,ones,blank)
return

' In order to avoid excessive current-draw by motors,
' always stop & pause a bit before changing directions.

frwrd: 										' go forward
gosub stopp
high dirL
high dirR
pwmduty spdL,speedL
pwmduty spdR,speedR	
return
	
left: 										' turn left about 90 degrees
gosub stopp
low  dirL
high dirR
pwmduty spdL,speedL
pwmduty spdR,speedR
pause abit
gosub stopp
return

right: 										' turn right about 90 degrees
gosub stopp
high dirL
low  dirR
pwmduty spdL,speedL
pwmduty spdR,speedR
pause abit
gosub stopp
return

rvrse: 										' go in reverse
gosub stopp
low dirL
low dirR
pwmduty spdL,speedL
pwmduty spdR,speedR
return

stopp: 										' stop both motors
pwmduty spdL,0
pwmduty spdR,0	  
wait 2
return